/***
*** gsc16AI64_ioctl.h  
***
***  General description of this file:
***     Device driver source code for General Standards 16AI64
***     family of analog input boards. This file is part of the Linux
***     driver source distribution for this board.
***     
***
***  Copyrights (c):
***     General Standards Corporation (GSC), 2004-2005
***
***  Author:
***     Evan Hillman (evan@generalstandards.com)
***
***  Support:
***     Primary support for this driver is provided by GSC. 
***
***/
#ifndef __DEVICE_IOCTL_H_INCLUDED__
#define __DEVICE_IOCTL_H_INCLUDED__

#define DEFAULT_TIMEOUT 10
#define DEFAULT_TIMEOUT_MSEC MSECS_TO_SLEEP(DEFAULT_TIMEOUT*1000)
 
#define IOCTL_DEVICE_MAGIC   's'

 /**
  ** IOCTL defines.  See below for API details.
  **
  **/

#define IOCTL_DEVICE_NO_COMMAND                _IO  (IOCTL_DEVICE_MAGIC,1)
#define IOCTL_DEVICE_INIT_BOARD                _IO  (IOCTL_DEVICE_MAGIC,2)
#define IOCTL_DEVICE_READ_REGISTER             _IOWR(IOCTL_DEVICE_MAGIC,3, struct device_register_params)  
#define IOCTL_DEVICE_WRITE_REGISTER            _IOWR(IOCTL_DEVICE_MAGIC,4, struct device_register_params)   
#define IOCTL_DEVICE_SS_MODE_CONFIG            _IOW (IOCTL_DEVICE_MAGIC,6, unsigned long)
#define IOCTL_DEVICE_ENABLE_DMA                _IOW (IOCTL_DEVICE_MAGIC,7, unsigned long)
#define IOCTL_DEVICE_GET_DEVICE_ERROR          _IOR (IOCTL_DEVICE_MAGIC,9, unsigned long)
#define IOCTL_DEVICE_INPUT_RANGE_CONFIG        _IOW (IOCTL_DEVICE_MAGIC,10,unsigned long)
#define IOCTL_DEVICE_INPUT_MODE_CONFIG         _IOW (IOCTL_DEVICE_MAGIC,11,unsigned long)
#define IOCTL_DEVICE_INPUT_FORMAT_CONFIG       _IOW (IOCTL_DEVICE_MAGIC,12,unsigned long)
#define IOCTL_DEVICE_START_AUTOCAL			   _IO  (IOCTL_DEVICE_MAGIC,13)
#define IOCTL_DEVICE_START_SINGLE_SCAN         _IO  (IOCTL_DEVICE_MAGIC,14)
#define IOCTL_DEVICE_SET_SCAN_SIZE             _IOW (IOCTL_DEVICE_MAGIC,15,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_CLOCK            _IOW (IOCTL_DEVICE_MAGIC,16,unsigned long)
#define IOCTL_DEVICE_DISABLE_PCI_INTERRUPTS    _IO  (IOCTL_DEVICE_MAGIC,17)
#define IOCTL_DEVICE_SET_B_CLOCK_SOURCE        _IOW (IOCTL_DEVICE_MAGIC,18,unsigned long)
#define IOCTL_DEVICE_SINGLE_CHAN_SELECT        _IOW (IOCTL_DEVICE_MAGIC,19,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_RATE_A           _IOR (IOCTL_DEVICE_MAGIC,20,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_RATE_B           _IOW (IOCTL_DEVICE_MAGIC,21,unsigned long)
#define IOCTL_DEVICE_GEN_DISABLE_A             _IO  (IOCTL_DEVICE_MAGIC,22)
#define IOCTL_DEVICE_GEN_DISABLE_B             _IO  (IOCTL_DEVICE_MAGIC,23)
#define IOCTL_DEVICE_GEN_ENABLE_A              _IO  (IOCTL_DEVICE_MAGIC,24)
#define IOCTL_DEVICE_GEN_ENABLE_B              _IO  (IOCTL_DEVICE_MAGIC,25)
#define IOCTL_DEVICE_SET_BUFFER_THRESHOLD      _IOW (IOCTL_DEVICE_MAGIC,26,unsigned long)
#define IOCTL_DEVICE_CLEAR_BUFFER              _IO  (IOCTL_DEVICE_MAGIC,27)
#define IOCTL_DEVICE_GET_DEVICE_TYPE           _IOR (IOCTL_DEVICE_MAGIC,28,unsigned long)
#define IOCTL_DEVICE_ENABLE_BUFFER_SS          _IO  (IOCTL_DEVICE_MAGIC,29)
#define IOCTL_DEVICE_DISABLE_BUFFER_SS         _IO  (IOCTL_DEVICE_MAGIC,30)
#define IOCTL_DEVICE_ENABLE_DMA_ALMOST_SS      _IO  (IOCTL_DEVICE_MAGIC,31)
#define IOCTL_DEVICE_DISABLE_DMA_ALMOST_SS     _IO  (IOCTL_DEVICE_MAGIC,32)
#define IOCTL_DEVICE_SET_TIMEOUT			   _IOW (IOCTL_DEVICE_MAGIC,33,unsigned long)
#define IOCTL_DEVICE_FILL_BUFFER			   _IOW (IOCTL_DEVICE_MAGIC,34,unsigned long)
#define IOCTL_DEVICE_DETECT_OVERFLOW		   _IOW (IOCTL_DEVICE_MAGIC,35,unsigned long)

#define IOCTL_DEVICE_SET_UNIPOLAR_INPUTS_SSA   _IOW (IOCTL_DEVICE_MAGIC,36,unsigned long)
#define IOCTL_DEVICE_SET_DIFF_PROCESSING_SSA   _IOW (IOCTL_DEVICE_MAGIC,37,unsigned long)
#define IOCTL_DEVICE_READ_BUFFER_UNDERFLOW_SSA _IOR (IOCTL_DEVICE_MAGIC,38,unsigned long)
#define IOCTL_DEVICE_READ_BUFFER_OVERFLOW_SSA  _IOR (IOCTL_DEVICE_MAGIC,39,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_MARKING_SSA      _IOW (IOCTL_DEVICE_MAGIC,40,unsigned long)
#define IOCTL_DEVICE_SET_DEMAND_MODE_DISABLE_SSA _IOW (IOCTL_DEVICE_MAGIC,41,unsigned long)
#define IOCTL_DEVICE_SET_THRESHOLD_4X_SSA      _IOW (IOCTL_DEVICE_MAGIC,42,unsigned long)
#define IOCTL_DEVICE_RESET_BUFFER_UNDERFLOW_SSA _IO (IOCTL_DEVICE_MAGIC,43)
#define IOCTL_DEVICE_RESET_BUFFER_OVERFLOW_SSA _IO (IOCTL_DEVICE_MAGIC,44)
#define IOCTL_DEVICE_SET_AUX_0_MODE_SSA        _IO (IOCTL_DEVICE_MAGIC,45)
#define IOCTL_DEVICE_SET_AUX_1_MODE_SSA        _IO (IOCTL_DEVICE_MAGIC,46)
#define IOCTL_DEVICE_SET_AUX_2_MODE_SSA        _IO (IOCTL_DEVICE_MAGIC,47)
#define IOCTL_DEVICE_SET_AUX_3_MODE_SSA        _IO (IOCTL_DEVICE_MAGIC,48)
#define IOCTL_DEVICE_SET_INPUT_INVERT_MODE_SSA _IO (IOCTL_DEVICE_MAGIC,49)
#define IOCTL_DEVICE_SET_OUTPUT_INVERT_MODE_SSA _IO (IOCTL_DEVICE_MAGIC,50)
#define IOCTL_DEVICE_SET_NOISE_SUPPRESSION_SSA _IO (IOCTL_DEVICE_MAGIC,51)
#define IOCTL_DEVICE_SET_HW_BUFFER_SIZE        _IOW (IOCTL_DEVICE_MAGIC,52,unsigned long)

/*****************************************************************************
 ** IOCTL_DEVICE_NO_COMMAND
 **
 ** Parameter = NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_INIT_BOARD
 **
 ** Parameter = NONE
 **/

/*****************************************************************************
** IOCTL_DEVICE_GET_DEVICE_TYPE	  	  
**
** returns an enumeration of the type of board the driver is attached to.
**
** Parameter = unsigned long *pType;
**/

enum type_index{
	gsc12ai64,
	gsc16ai64,
	gsc16ai64ss
};

/*****************************************************************************
 ** IOCTL_DEVICE_READ_REGISTER	  
 ** IOCTL_DEVICE_WRITE_REGISTER	  
 **
 ** Parameter = DEVICE_REGISTER_PARAMS *pRegParams;
 **
 ** This structure is used to store information about a register. The
 ** IOCTL_DEVICE_READ_REGISTER and IOCTL_DEVICE_WRITE_REGISTER ioctl commands use this
 ** structure to read a particular register and to write a value into a particular
 ** register. 'DeviceRegister' stores the index of the register, range being 0-94, and
 ** 'ulRegisterValue' stores the register value, range being 0x0-0xFFFFFFFF.
 **/
typedef struct device_register_params {
  unsigned int DeviceRegister;	      	    /* range: 0-94, see definitions below */
  unsigned long ulValue;	    /* range: 0x0-0xFFFFFFFF, same register values below */
} DEVICE_REGISTER_PARAMS, *PDEVICE_REGISTER_PARAMS;


/* ==== Register Offsets ==== */
/* ---- 'local' Registers ---- */

#define BOARD_CTRL_REG			0
#define INTERRUPT_CONTROL_REG	1
#define INPUT_DATA_BUFF_REG		2
#define INPUT_BUFF_CTRL_REG		3
#define RATE_A_GENERATOR_REG	4
#define RATE_B_GENERATOR_REG	5
#define BUFFER_SIZE_REG			6
#define RESERVED_1_REG			7
#define SCAN_SYNCH_CTRL_REG		8
#define RESERVED_2_REG			9
#define FIRMWARE_REV_REG		10
#define AUTOCAL_VALUES_REG		11
#define AUX_R_W_REG             12
#define AUX_SYNCH_IO_CTRL       13
#define SCAN_MARK_FIRST_VAL     14
#define SCAN_MARK_SECOND_VAL    15


/* ---- DMA Registers ---- */
/* ---- see PLX documentation for info ---- */

#define DMA_CH_0_MODE                	    32
#define DMA_CH_0_PCI_ADDR            	    33
#define DMA_CH_0_LOCAL_ADDR          	    34
#define DMA_CH_0_TRANS_BYTE_CNT      	    35
#define DMA_CH_0_DESC_PTR            	    36
#define DMA_CH_1_MODE                	    37
#define DMA_CH_1_PCI_ADDR            	    38
#define DMA_CH_1_LOCAL_ADDR          	    39
#define DMA_CH_1_TRANS_BYTE_CNT      	    40
#define DMA_CH_1_DESC_PTR            	    41
#define DMA_CMD_STATUS               	    42
#define DMA_MODE_ARB_REG             	    43
#define DMA_THRESHOLD_REG            	    44

/* ---- PCI Configuration Registers ---- */
/* ---- see PLX documentation for info ---- */

#define DEVICE_VENDOR_ID              	    0
#define STATUS_COMMAND                      1
#define CLASS_CODE_REVISION_ID              2
#define BIST_HDR_TYPE_LAT_CACHE_SIZE        3
#define PCI_MEM_BASE_ADDR                   4
#define PCI_IO_BASE_ADDR                    5
#define PCI_BASE_ADDR_0                     6
#define PCI_BASE_ADDR_1                     7
#define CARDBUS_CIS_PTR                     10
#define SUBSYS_ID_VENDOR_ID                 11
#define PCI_BASE_ADDR_LOC_ROM               12
#define LAT_GNT_INT_PIN_LINE                15

/* ---- Local Configuration Registers ---- */
/* ---- see PLX documentation for info ---- */

#define PCI_TO_LOC_ADDR_0_RNG               0
#define LOC_BASE_ADDR_REMAP_0               1
#define MODE_ARBITRATION                    2
#define BIG_LITTLE_ENDIAN_DESC              3
#define PCI_TO_LOC_ROM_RNG                  4
#define LOC_BASE_ADDR_REMAP_EXP_ROM         5
#define BUS_REG_DESC_0_FOR_PCI_LOC          6
#define DIR_MASTER_TO_PCI_RNG               7
#define LOC_ADDR_FOR_DIR_MASTER_MEM         8
#define LOC_ADDR_FOR_DIR_MASTER_IO          9
#define PCI_ADDR_REMAP_DIR_MASTER           10
#define PCI_CFG_ADDR_DIR_MASTER_IO          11
#define PCI_TO_LOC_ADDR_1_RNG               92
#define LOC_BASE_ADDR_REMAP_1               93
#define BUS_REG_DESC_1_FOR_PCI_LOC          94

/* ---- Run Time Registers ---- */
/* ---- see PLX documentation for info ---- */

#define MAILBOX_REGISTER_0                  16
#define MAILBOX_REGISTER_1                  17
#define MAILBOX_REGISTER_2                  18
#define MAILBOX_REGISTER_3                  19
#define MAILBOX_REGISTER_4                  20
#define MAILBOX_REGISTER_5                  21
#define MAILBOX_REGISTER_6                  22
#define MAILBOX_REGISTER_7                  23
#define PCI_TO_LOC_DOORBELL                 24
#define LOC_TO_PCI_DOORBELL                 25
#define INT_CTRL_STATUS                     26
#define PROM_CTRL_CMD_CODES_CTRL            27
#define DEVICE_ID_VENDOR_ID                 28
#define REVISION_ID                         29
#define MAILBOX_REG_0                       30
#define MAILBOX_REG_1                       31

/* ---- Messaging Queue Registers ---- */
/* ---- see PLX documentation for info ---- */

#define OUT_POST_Q_INT_STATUS               12
#define OUT_POST_Q_INT_MASK                 13
#define IN_Q_PORT                           16
#define OUT_Q_PORT                          17
#define MSG_UNIT_CONFIG                     48
#define Q_BASE_ADDR                         49
#define IN_FREE_HEAD_PTR                    50
#define IN_FREE_TAIL_PTR                    51
#define IN_POST_HEAD_PTR                    52
#define IN_POST_TAIL_PTR                    53
#define OUT_FREE_HEAD_PTR                   54
#define OUT_FREE_TAIL_PTR                   55
#define OUT_POST_HEAD_PTR                   56
#define OUT_POST_TAIL_PTR                   57
#define Q_STATUS_CTRL_REG                   58

/* ==== Values for some key registers ==== */

/* ---- masks for the 'local' Board Control Register (32-bit addr 0). ---- */
/* ---- see GSC documentation for info ---- */

// analog input mode

#define BCR_DIFFERENTIAL 0
#define BCR_SINGLE_ENDED 1
#define BCR_ZERO_TEST 2
#define BCR_VREF_TEST 3

#define BCR_AIM_SHIFT                        0
#define BCR_AIM_MASK                         0x0007

// analog input range

#define BCR_RANGE_SHIFT                      4
#define BCR_RANGE_MASK                       (3<<BCR_RANGE_SHIFT)

#define BCR_RANGE_2_5						(0)
#define BCR_RANGE_5	  					    (1)
#define BCR_RANGE_10 						(2)

// unipolar inputs - SSA board only.

#define BCR_UNIPOLAR_INPUTS_SSA             (1<<3)

// offset binary mode 

#define BCR_OFFSET_BINARY_SHIFT              6
#define BCR_FORMAT_MASK                      (1<<BCR_OFFSET_BINARY_SHIFT)

// External synch

#define BCR_ENABLE_EXTERN_SYNC               (1<<7)

// Differential processing modes. SSA boards only.  Choices of:
// - Single ended.
// - Pseudo-differential.
// - Full differential.

#define BCR_DIFF_PROC_SHIFT_SSA             8
#define BCR_DIFF_PROC_MASK_SSA              (3<<BCR_DIFF_PROC_SHIFT_SSA)

#define BCR_DIFF_PROC_SE_SSA                0
#define BCR_DIFF_PROC_PSEUDO_DIFF_SSA       1
#define BCR_DIFF_PROC_FULL_DIFF_SSA         2

// start single scan

#define BCR_INPUT_SYNCH_START				(1<<12)

// init autocalibration

#define BCR_AUTOCAL                         (1<<13)

// autocalibration test passed flag

#define BCR_AUTOCAL_PASS                    (1<<14)

// initialize the hardware

#define BCR_INITIALIZE                      (1<<15)

// Buffer underflow flag

#define BCR_BUFFER_UNDERFLOW_SSA            (1<<16)

// Buffer overflow flag

#define BCR_BUFFER_OVERFLOW_SSA             (1<<17)

// Enables scan marking.  Refer to hardware manual.

#define ENABLE_SCAN_MARKING_SSA             (1<<18)

// Disables demand-mode DMA

#define DISABLE_DEMAND_MODE_SSA             (1<<19)

/* ---- masks for the 'local' Interrupt Control Register (32-bit addr 1). ---- */
/* ---- see GSC documentation for info ---- */

#define ICR_GROUP_0_MASK        0x07
#define ICR_GROUP_0_SHIFT       0
#define ICR_GROUP_0_REQUEST     (1<<3)

#define ICR_GROUP_0_IDLE 0
#define ICR_GROUP_0_AUTOCAL_COMPLETE    1
#define ICR_GROUP_0_INPUT_SCAN_INIT     2
#define ICR_GROUP_0_INPUT_SCAN_COMPLETE 3

#define ICR_GROUP_1_MASK        0x07
#define ICR_GROUP_1_SHIFT       4
#define ICR_GROUP_1_REQUEST     (1<<7)

#define ICR_GROUP_1_IDLE    0
#define ICR_GROUP_1_LO_HI   1
#define ICR_GROUP_1_HI_LO   2

/* ---- masks for the 'local' Input Data Register (32-bit addr 2). ---- */
/* ---- see GSC documentation for info ---- */

#define VALID_DATA_MASK     0xFF
#define CHANNEL_0_FLAG      (1<<16)

// SSA specific data masks for "packed" data.

#define SCAN_MARKER_DEFAULT_SSA 0x00
#define LO_DATA_MASK_SSA        0xFF
#define HI_DATA_SHIFT_SSA       16
#define HI_DATA_MASK_SSA        (LO_DATA_MASK_SSA<<HI_DATA_SHIFT_SSA)
#define PACKED_DATA_MASK_SSA    0xFFFF

/* ---- masks for the 'local' Input Data Control Registers (32-bit addr 3). ---- */
/* ---- see GSC documentation for info ---- */

#define IDC_INPUT_CLEAR_BUFFER  (1<<16)
#define IDC_THRESHOLD_FLAG      (1<<17)
#define IDC_INPUT_BUFFER_SIZE   0xFFFF 
#define IDC_THRESHOLD_MASK      IDC_INPUT_BUFFER_SIZE

// the maximum size for the hardware buffer.  Used for error checking.
#define MAX_BUFFER_SIZE 512*1024

// these for the SS boards only 

#define IDC_DISABLE_BUFFER_SS               (1<<18)
#define IDC_TERMINATE_DMA_ALMOST_EMPTY_SS   (1<<19)

// These for the SSA board only

#define IDC_THRESHOLD_4X_SSA        (1<<19)

/* ---- masks for the 'local' Rate A Generator Register (32-bit addr 4). ---- */
/* ---- masks for the 'local' Rate B Generator Register (32-bit addr 5). ---- */
/* ---- see GSC documentation for info ---- */

#define RGR_NRATE_MASK (0xFFFF)
#define RGR_GEN_DISABLE (1<<16)

/* ---- masks for the 'local' Buffer Size (32-bit addr 6). ---- */
/* ---- see GSC documentation for info ---- */

/* no masks required... */

/* ---- masks for the 'local' Scan and Synch Control Register(32-bit addr 8). ---- */
/* ---- see GSC documentation for info ---- */

// Scan size defines

#define SSCR_1_CHAN_SIZE 0
#define SSCR_2_CHAN_SIZE 1
#define SSCR_4_CHAN_SIZE 2
#define SSCR_8_CHAN_SIZE 3
#define SSCR_16_CHAN_SIZE 4
#define SSCR_32_CHAN_SIZE 5
#define SSCR_64_CHAN_SIZE 6

#define SSCR_CHAN_SIZE_MASK 0x07

// analog inputs scan clock

#define SSCR_CLOCK_SHIFT 3
#define SSCR_SCAN_CLOCK_MASK (3<<SSCR_CLOCK_SHIFT)

#define SSCR_INTERNAL_RATE_A 0
#define SSCR_INTERNAL_RATE_B (1)
#define SSCR_EXTERNAL_SYNCH  (2)
#define SSCR_BCR_INPUT_SYNCH (3)

// single channel defines

#define SSCR_SINGLE_CHANNEL_SHIFT (12)
#define SSCR_SINGLE_CHANNEL_MASK (0x3F<<12)

// Rate "B" clock source

#define SSCR_RATE_B_CLOCK_SOURCE_SHIFT (10)
#define SSCR_RATE_B_CLOCK_SOURCE_MASK (1<<SSCR_RATE_B_CLOCK_SOURCE_SHIFT)

#define SSCR_MASTER_CLOCK 0
#define SSCR_RATE_A_GENERATOR (1)

/* ---- Auxiliary Sync I/O Control ---- */

#define ASC_MODE_INACTIVE       0
#define ASC_MODE_ACTIVE_IN      1
#define ASC_MODE_ACTIVE_OUT     2

#define ASC_MODE_MASK 0x03

#define ASC_CONTROL_MODE_0_SHIFT 0


#define ASC_MODE_INACTIVE_0     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_0_SHIFT)
#define ASC_MODE_ACTIVE_IN_0    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_0_SHIFT)
#define ASC_MODE_ACTIVE_OUT_0   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_0_SHIFT)

#define ASC_CONTROL_MODE_1_SHIFT 2

#define ASC_MODE_INACTIVE_1     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_1_SHIFT)
#define ASC_MODE_ACTIVE_IN_1    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_1_SHIFT)
#define ASC_MODE_ACTIVE_OUT_1   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_1_SHIFT)

#define ASC_CONTROL_MODE_2_SHIFT 4

#define ASC_MODE_INACTIVE_2     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_2_SHIFT)
#define ASC_MODE_ACTIVE_IN_2    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_2_SHIFT)
#define ASC_MODE_ACTIVE_OUT_2   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_2_SHIFT)

#define ASC_CONTROL_MODE_3_SHIFT 6

#define ASC_MODE_INACTIVE_3     (ASC_MODE_INACTIVE<<ASC_CONTROL_MODE_3_SHIFT)
#define ASC_MODE_ACTIVE_IN_3    (ASC_MODE_ACTIVE_IN<<ASC_CONTROL_MODE_3_SHIFT)
#define ASC_MODE_ACTIVE_OUT_3   (ASC_MODE_ACTIVE_OUT<<ASC_CONTROL_MODE_3_SHIFT)

#define ASC_INVERT_INPUTS       (1<<8)
#define ASC_INVERT_OUTPUTS      (1<<9)
#define ASC_NOISE_SUPPRESS      (1<<10)

/* ---- DMA command/status register bits for channel 0 ---- */
/* ---- see PLX documentation for info ---- */

#define DMA0_ENABLE			    0x0001
#define DMA0_START			    0x0002
#define DMA0_ABORT			    0x0004
#define DMA0_CLR_INT		    0x0008
#define DMA0_DONE			    0x0010

/* ---- input buffer size ---- */

#define INPUT_BUFFER_SIZE (64*1024) 

/*****************************************************************************
 ** IOCTL_DEVICE_ENABLE_DMA
 **
 ** Parameter = unsigned long *DMAmode;
 **
 ** Set up to use DMA for data transfer.
 **/

enum {
	DMA_DISABLE,
	DMA_ENABLE,
	DMA_DEMAND_MODE
};

/*****************************************************************************
 ** IOCTL_DEVICE_GET_DEVICE_ERROR
 **
 ** Parameter = unsigned long *pulDeviceError;
 ** Return value range: 0-10, see codes below
 **/

#define DEVICE_SUCCESS                         0
#define DEVICE_INVALID_PARAMETER               1
#define DEVICE_INVALID_BUFFER_SIZE             2
#define DEVICE_PIO_TIMEOUT                     3
#define DEVICE_DMA_TIMEOUT                     4
#define DEVICE_IOCTL_TIMEOUT                   5
#define DEVICE_OPERATION_CANCELLED             6
#define DEVICE_RESOURCE_ALLOCATION_ERROR       7
#define DEVICE_INVALID_REQUEST                 8
#define DEVICE_AUTOCAL_FAILED    	      	   9
#define DEVICE_BUFFER_OVERFLOW    	      	   10

/*****************************************************************************
 ** IOCTL_DEVICE_INPUT_RANGE_CONFIG
 **
 ** Parameter = unsigned long *pInputRange;
 ** Range: 0-2, see codes below
 **/

/* ---- scan mode codes ---- */
#define INPUT_RANGE_2_5 0
#define INPUT_RANGE_5   1
#define INPUT_RANGE_10  3

/*****************************************************************************
 **  IOCTL_DEVICE_INPUT_MODE_CONFIG
 **
 **  Parameter = unsigned long *iMode
 **/

/* ---- mode config codes ---- */
#define INPUT_MODE_SHIFT	0
#define INPUT_MODE_MASK		0x07

#define INPUT_DIFFERENTIAL  0
#define INPUT_SINGLE_ENDED  1
#define INPUT_ZERO_TEST     2
#define INPUT_VREF_TEST     3
#define INPUT_SYSTEM_ANALOG_SS 0

/*****************************************************************************
** IOCTL_DEVICE_SS_MODE_CONFIG
**
**  Parameter = unsigned long *iMode
**/
#define SS_SINGLE_ENDED			0
#define SS_PSEUDO_DIFFERENTIAL	1
#define SS_FULL_DIFFERENTIAL	3

#define SS_MODE_SHIFT 8
#define SS_MODE_MASK (3 << 8)

/*****************************************************************************
 **  IOCTL_DEVICE_INPUT_FORMAT_CONFIG
 **
 **  Parameter = unsigned long *format
 ** Range: 0-3, see codes below
 **/

/* ---- device test codes ---- */
#define FORMAT_TWO_COMPLEMENT     0
#define FORMAT_OFFSET_BINARY      1

/*****************************************************************************
 **  IOCTL_DEVICE_START_AUTOCAL
 **
 **  Parameter =  NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_START_SINGLE_SCAN
 **
 ** Parameter = NONE;
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_SIZE
 **
 ** Parameter = unsigned long *pScanSize;
 ** Range: 0-6, see codes below
 **/

/* ---- scan size codes ---- */
#define SCAN_SINGLE_CHAN	0
#define SCAN_2_CHAN         1
#define SCAN_4_CHAN		    2
#define SCAN_8_CHAN		    3
#define SCAN_16_CHAN		4
#define SCAN_32_CHAN		5
#define SCAN_64_CHAN		6

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_CLOCK
 **
 ** select the analog input scan clock
 **
 ** Parameter = unsigned long *clock
 ** range=0-3
 **/

#define INTERNAL_RATE_A	0
#define INTERNAL_RATE_B	1
#define EXTERNAL_SYNCH	2
#define BCR_INPUT_SYNCH	3

/*****************************************************************************
 **  IOCTL_DEVICE_DISABLE_PCI_INTERRUPTS
 **
 **  Parameter = NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_B_CLOCK_SOURCE
 **
 ** Parameter = unsigned long *pSource;
 ** Range: 0-1, see codes below
 **/

#define MASTER_CLOCK     0
#define RATE_A_OUTPUT    1

/*****************************************************************************
 ** IOCTL_DEVICE_SINGLE_CHAN_SELECT
 **
 ** Parameter =  unsigned long *channel;
 ** Range: 0-63
 **
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_RATE_A
 **
 ** Parameter = unsigned long *setting;
 ** Range = 0-0xFFFF
 **
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_RATE_B
 **
 ** Parameter = unsigned long *setting;
 ** Range = 0-0xFFFF
 **
 **/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_DISABLE_A
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_DISABLE_B
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_ENABLE_A
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_ENABLE_B
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_SET_BUFFER_THRESHOLD
**
**  Parameter = unsigned long * threshold
**  Range = 0..0xFFFF
**/

/*****************************************************************************
**  IOCTL_DEVICE_CLEAR_BUFFER
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_ENABLE_BUFFER_SS
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_DISABLE_BUFFER_SS
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_ENABLE_DMA_ALMOST_SS
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_DISABLE_DMA_ALMOST_SS
**
**  Parameter = NONE
**/

/*************************************************************************
** IOCTL_DEVICE_FILL_BUFFER
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is used to tell the driver to fill the user buffer before returning.
** If set, the driver will make one or more read transfers from the hardware to 
** satisfy the user request.  If set to FALSE, the driver will return one or more samples
** per the Linux convention.
**
**/

/*************************************************************************
** IOCTL_DEVICE_DETECT_OVERFLOW
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is used to tell the driver to fail the read operation if the
** input buffer is found to be full.  This allows the user to detect data loss. 
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_UNIPOLAR_INPUTS_SSA
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is select unipolar or bipolar inputs.  Possible values are: 
**
**      BIPOLAR_INPUTS
**      UNIPOLAR_INPUTS 
**
**/

#define BIPOLAR_INPUTS  0
#define UNIPOLAR_INPUTS BCR_UNIPOLAR_INPUTS_SSA

/*************************************************************************
** IOCTL_DEVICE_SET_DIFF_PROCESSING_SSA
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is used to set the differential input processing mode.  Possible
** values are: 
**
**      SINGLE_ENDED_PROC                       
**      PSEUDO_DIFFERETIAL_PROC        
**      FULL_DIFFERENTIAL_PROC           
**
**/

#define SINGLE_ENDED_PROC       BCR_DIFF_PROC_SE_SSA                
#define PSEUDO_DIFFERETIAL_PROC BCR_DIFF_PROC_PSEUDO_DIFF_SSA       
#define FULL_DIFFERENTIAL_PROC  BCR_DIFF_PROC_FULL_DIFF_SSA         

/*************************************************************************
** IOCTL_DEVICE_READ_BUFFER_UNDERFLOW_SSA
**
** Parameter: unsigned long *state
** 
** Return: non-zero if buffer underflow detected, zero otherwise.
**
** This ioctl is used to read the state of the buffer underflow flag in 
** the BCR register. 
**
**/

/*************************************************************************
** IOCTL_DEVICE_READ_BUFFER_OVERFLOW_SSA 
**
** Parameter: unsigned long *state
** 
** Return: non-zero if buffer underflow detected, zero otherwise.
**
** This ioctl is used to read the state of the buffer overflow flag in 
** the BCR register. 
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_SCAN_MARKING_SSA   
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is used to enable or disable scan marking.  Pass TRUE to
** enable scan marking, FALSE to disable scan marking.  Default is
** disabled.
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_DEMAND_MODE_DISABLE_SSA 
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is used to tell the hardware. Pass TRUE to
** Disable demand mode, FALSE to enable demand mode.  Default is
** demand mode enabled (FALSE).
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_THRESHOLD_4X_SSA     
**
** Parameter: unsigned long *state
** 
** Return: none
**
** This ioctl is used set the 4X threshold mode.  pass TRUE to enable
** 4X threshold, pass FALSE to enable normal threshold handling.  
** Default is normal (FALSE).
**
**/

/*************************************************************************
** IOCTL_DEVICE_RESET_BUFFER_UNDERFLOW_SSA     
**
** Parameter: none
** 
** Return: none
**
** This ioctl resets the buffer underflow bit in the BCR register.
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_AUX_0_MODE_SSA     
**
** Parameter: unsigned long *state 
** 
** Return: none
**
** This ioctl sets the AUX I/O mode for channel 0. Aux I/O control mode.  
**
**  Possible values are:
**
**      AUX_INACTIVE
**      AUX_ACTIVE_IN
**      AUX_ACTIVE_OUT
**
**/
#define AUX_INACTIVE    0     
#define AUX_ACTIVE_IN   1    
#define AUX_ACTIVE_OUT  2   

/*************************************************************************
** IOCTL_DEVICE_SET_AUX_1_MODE_SSA     
**
** Parameter: unsigned long *state 
**
**  AUX_INACTIVE
**  AUX_ACTIVE_IN
**  AUX_ACTIVE_OUT
** 
** Return: none
**
** This ioctl sets the AUX I/O mode for channel 1.
**
**  Possible values are:
**
**      AUX_INACTIVE
**      AUX_ACTIVE_IN
**      AUX_ACTIVE_OUT
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_AUX_2_MODE_SSA     
**
** Parameter: unsigned long *state 
**
**  AUX_INACTIVE
**  AUX_ACTIVE_IN
**  AUX_ACTIVE_OUT
** 
** Return: none
**
** This ioctl sets the AUX I/O mode for channel 2.
**
**  Possible values are:
**
**      AUX_INACTIVE
**      AUX_ACTIVE_IN
**      AUX_ACTIVE_OUT
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_AUX_3_MODE_SSA     
**
** Parameter: unsigned long *state 
**
**  AUX_INACTIVE
**  AUX_ACTIVE_IN
**  AUX_ACTIVE_OUT
** 
** Return: none
**
** This ioctl sets the AUX I/O mode for channel 3.
**
**  Possible values are:
**
**      AUX_INACTIVE
**      AUX_ACTIVE_IN
**      AUX_ACTIVE_OUT
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_INPUT_INVERT_MODE_SSA 
**
** Parameter: Select inverted or non-inverted aux inputs.  Possible values are:
**
**  FALSE - Inputs detected on low-to-high transition.
**  TRUE - Inputs are detected on the high-to-low transition.
** 
** Return: none
**
**/

/*************************************************************************
** IOCTL_DEVICE_SET_OUTPUT_INVERT_MODE_SSA 
**
**  Parameter: unsigned long * 
**  Return: none
**
** Select inverted or non-inverted aux outputs.  Possible values are:
**
**  FALSE - Active outputs produce low output pulse.
**  TRUE - Active outputs produce high output pulse.
** 
**/

/*************************************************************************
** IOCTL_DEVICE_SET_NOISE_SUPPRESSION_SSA
**
** Parameter: unsigned long *
** 
** Return: none
**
** This IOCTL is used to enable or disable noise suppression.  Default
** is noise suppression disabled.  Possible values are:
**
**      TRUE - Enable noise suppression
**      FALSE - Disable noise suppression
**/

/*************************************************************************
** IOCTL_DEVICE_SET_HW_BUFFER_SIZE
**
** Parameter: unsigned long *
** 
** Return: none
**
** This IOCTL is set the size of the hardware FIFO for the driver.  The
** size is used by the driver to detect overflow.  This does not change
** anything about the hardware.  Range of values are:
** 
**      0-MAX_BUFFER_SIZE (512K)
**
**/

#endif	/* entire file */

