/***
*** hardware.c  
***
***  General description of this file:
***     Device driver source code for General Standards 16AI64
***     family of analog input boards. This file is part of the Linux
***     driver source distribution for this board.
***     
***
***  Copyrights (c):
***     General Standards Corporation (GSC), 2004-2005
***
***  Author:
***     Evan Hillman (evan@generalstandards.com)
***
***  Support:
***     Primary support for this driver is provided by GSC. 
***
***/
//////////////////////////////////////////////////////////////////////////
// set the following flag to trace interrupt debug messages.
#ifdef DEBUG
#define TRACE_LOCAL TRUE
#endif

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/pci.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/timer.h>
#include <linux/ioctl.h>

#include <asm/io.h>
#include <asm/system.h>
#include <asm/uaccess.h>

#include "sysdep.h"
#include "gsc16ai64_ioctl.h"
#include "internals.h"

#include "plx_regs.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
void DisableIrqPlx(struct device_board *device)
{
    writel(1, IntCntrlStat(device));
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void EnableIrqPlxLocal(struct device_board *device)
{
    writel(readl(IntCntrlStat(device)) | IRQ_PCI_ENABLE | IRQ_LOCAL_PCI_ENABLE, IntCntrlStat(device));
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void DisableIrqLocalAll(struct device_board *device)
{
    writelocal(device,0,INTERRUPT_CONTROL_REG);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void EnableIrqPlxDMA(struct device_board *device)
{
    writel(readl(IntCntrlStat(device)) | IRQ_PCI_ENABLE | IRQ_DMA_0_ENABLE, IntCntrlStat(device));
}

static int groupMasks[] =
{
    ICR_GROUP_0_MASK<<ICR_GROUP_0_SHIFT,
        ICR_GROUP_1_MASK<<ICR_GROUP_1_SHIFT,
};

static int groupShifts[] =
{
    ICR_GROUP_0_SHIFT,
        ICR_GROUP_1_SHIFT,
};


/************************************************************************/
/*                                                                      */
/************************************************************************/
void EnableIrqLocal(struct device_board *device, int group, u32 irq)
{
    u32 regval;

    regval = readlocal(device,INTERRUPT_CONTROL_REG);
    regval &= (~(groupMasks[group]));
    regval |= (irq << groupShifts[group]);
    writelocal(device,regval,INTERRUPT_CONTROL_REG);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void DisableIrqLocal(struct device_board *device, int group, u32 irq)
{
    u32 regval;

    regval = readlocal(device,INTERRUPT_CONTROL_REG);
    regval &= (~(groupMasks[group]));
    writelocal(device,regval,INTERRUPT_CONTROL_REG);
}

/************************************************************************/
/* writelocal                                                           */
/*                                                                      */
/* Write to the board local registers, whether the local registers      */
/* are in I/O space or memory space.                                    */
/*                                                                      */
/*                                                                      */
/************************************************************************/
void writelocal(struct device_board *device, unsigned value, unsigned address)
{
    if (device->PciBar[2].IsIoMapped) /* i/o space */
    {
        //msg("writelocal I/O space addr: %.8X value: %.8X.\n",address,value);
        outl(value, (unsigned) device->local_addr+address*4);
    }
    else
    {
        writel(value, device->local_addr+address);
    }
};

/************************************************************************/
/* readlocal                                                            */
/*                                                                      */
/* Read from the board local registers, whether the local registers     */
/* are in I/O space or memory space.                                    */
/*                                                                      */
/*                                                                      */
/************************************************************************/
unsigned readlocal(struct device_board *device, unsigned address)
{
    if (device->PciBar[2].IsIoMapped) /* i/o space */
    {
        //      msg("readlocal I/O space addr: %.8X \n",address);
        return inl((unsigned)device->local_addr+address*4);
    }
    else
    {
        return readl(device->local_addr+address);
    }
    return 0;
};

