 #include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>

#include "gsc16AI64_ioctl.h"

#define MAX_CHANNELS 64

#define BUFFER_SIZE (1024*32) // rather arbitrary

char driver_name[80]="/dev/gsc16ai64\0";

/* text names to describe each type of board supported. */
char *boards_supported[] = {
	"PMC-12AI64", /* PMC-12AI64 */
	"PMC-16AI64",	/* PMC-16AI64 */
	"PMC-16AI64SS",	/* PMC-16AI64SS */
	"null"          /* null terminated list.... */
};

/* number of channels for each board supported. */

int channels[] = {
	64, /* PMC-12AI64 */
	64, /* PMC-16AI64 */
	64  /* PMC-16AI64SS */
};

int main(int argc, char *argv[])
{
	struct device_register_params rw_q;
	unsigned long total;
	unsigned long buffer[BUFFER_SIZE];
	unsigned long count[MAX_CHANNELS];
	int board_type;
	int fd, res=0;
	int print, nread, i;
	unsigned long setting;
	int BuffersRead;
	int samples;

	if (argc < 2) {
		printf("%s: missing argument\nusage: testapp [0|1|2|...]\n", argv[0]);
		return (1);
	}

	printf("about to open\n");
	strcat(driver_name, argv[1]);
	fd = open(driver_name, O_RDWR);

	if (fd < 0) {
		printf("%s: can not open device %s\n", argv[0], argv[1]);
		return (1);
	}

	/*
	*	init the board.
	*/
	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_GET_DEVICE_TYPE, &board_type);
	if (res < 0) {
		printf("%s: ioctl IOCTL_SDI_GET_DEVICE_TYPE failed\n", argv[0]);
		return (1);
	}

	printf("open OK - board type: %s channels: %d\n", boards_supported[board_type],channels[board_type]);
	rw_q.DeviceRegister = 0x0;
	rw_q.ulValue = 0x2000;
	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_READ_REGISTER, &rw_q);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_READ_REGISTER failed\n", argv[0]);
		return (1);
	}
	printf("before init: BCR is 0x%lx\n", rw_q.ulValue);
	res = ioctl(fd, IOCTL_DEVICE_INIT_BOARD, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_INIT_BOARD failed\n", argv[0]);
		return (1);
	}
	printf("board reset OK\n");
	rw_q.DeviceRegister = 0x0;
	rw_q.ulValue = 0x2000;
	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_READ_REGISTER, &rw_q);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_READ_REGISTER failed\n", argv[0]);
		return (1);
	}

	printf("after init: BCR is 0x%lx\n", rw_q.ulValue);

	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_START_AUTOCAL, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_START_AUTOCAL failed\n", argv[0]);
		return (1);
	}

	printf("auto calibration OK\n");

	ioctl(fd, IOCTL_DEVICE_CLEAR_BUFFER, NULL);
	/*
	 *	ensure that the acquisiton hardware is stopped by selecting
	 *  board control register clocking.
	 */
	ioctl(fd, IOCTL_DEVICE_GEN_DISABLE_A, NULL);
	ioctl(fd, IOCTL_DEVICE_GEN_DISABLE_B, NULL);
	/*
	*	set up input range and format and number of channels
	*/
	setting = INPUT_RANGE_10;
	ioctl(fd, IOCTL_DEVICE_INPUT_RANGE_CONFIG, &setting);

	setting = INPUT_SINGLE_ENDED;
	ioctl(fd, IOCTL_DEVICE_INPUT_MODE_CONFIG, &setting);

	setting = SCAN_64_CHAN;
	ioctl(fd, IOCTL_DEVICE_SET_SCAN_SIZE, &setting);
	
	setting = DMA_DISABLE;
	//setting = DMA_ENABLE;
	//setting = DMA_DEMAND_MODE;
	ioctl(fd, IOCTL_DEVICE_ENABLE_DMA, &setting);

	setting = 0x2000;
	ioctl(fd, IOCTL_DEVICE_SET_BUFFER_THRESHOLD, &setting);
	
//
	// start the hardware
	//
	ioctl(fd, IOCTL_DEVICE_GEN_ENABLE_A, NULL);

	//
	// set up for looping reading data.
	//
	for(i = 0; i < MAX_CHANNELS; i++) {
		count[i] = 0;
	}
	total = 0L;
	print = -1;
	nread = 0;
	printf("\n");
	BuffersRead=0;

	while (BuffersRead<10) {
		int channel;
		memset(buffer,0,sizeof(buffer));
		res = read(fd, buffer, BUFFER_SIZE);

		BuffersRead++;
		if (BuffersRead>10) {
			goto out;
		}

		if (res <= 0){
			printf("read error...res= %d\n",res);
			return (1);
		}
		samples = res / sizeof(long);
		total += (samples);
		nread++;
		channel=0;

		//printf("Got a buffer... size: %d\n",res);
		for(i = 0; i <samples; i++) {
			if (buffer[i]&CHANNEL_0_FLAG)
			{
				channel=0;
			}

			if (channel<0||channel>MAX_CHANNELS) {
				printf("Illegal channel value! %d\n",channel);
				goto out;
			}
			count[channel]++;
			channel++;
		}
		if ((total >> 20) != print) {
			print = (total >> 20);
			rw_q.DeviceRegister = BUFFER_SIZE_REG;
			res = ioctl(fd, (unsigned long)IOCTL_DEVICE_READ_REGISTER, &rw_q);
			printf("\rT=%ldM, A=%ld, F=%ld, C[i]=", (total >> 20), (total / nread), rw_q.ulValue);
			for(i = 0; i < 16; i++) {
				printf(" %ld",(count[i] >> 20));
			}
			printf("      \r");
			fflush(stdout);
		}
	}
out:
	for (i=0;i<MAX_CHANNELS;i++)
	{
		printf("%.8d ",count[i]);
		if ((i>0)&(i%8==0))
			printf("\n");
	}
	printf("\n");
	close(fd);
	return 0;
}
